from django.contrib import admin

from forms import AlertForm
from models import Alert
from utils import get_message_handler


class AlertAdmin(admin.ModelAdmin):
    form = AlertForm
    list_display = ('pk', 'level', 'tag_display', 'recipient', 'subject', 'is_read', 'is_email_sent', 'is_pushed', 'created')
    list_filter = ('level', 'tag')

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "tag":
            kwargs['choices'] = self.tag_choices()
        return super(AlertAdmin, self).formfield_for_choice_field(db_field, request, **kwargs)

    def tag_choices(self):
        message_handler = get_message_handler()
        return getattr(message_handler, 'TAGS', None)

    def tag_display(self, alert):
        choices = self.tag_choices()
        choice = dict(choices).get(alert.tag, None)
        if choice is not None:
            return choice
        return alert.tag

admin.site.register(Alert, AlertAdmin)



from models import Alert


def notify(recipient, tag, subject=None, context=None, level=None):
    kwargs = {
        'recipient': recipient,
        'tag': tag,
        'subject': subject,
        'context': context
    }
    if level is not None:
        kwargs['level'] = level
    return Alert.objects.create(**kwargs)


def mark_as_read(tag, user, subject=None, context=None):
    alerts = Alert.objects.by_tag(tag)
    if subject:
        alerts = alerts.of_subject(subject)
    if context:
        alerts = alerts.in_context(context)
    alerts.mark_as_read(user)


class AlertMessageHandler(object):
    def get_message(self, alert):
        raise NotImplementedError()



from django import template
from django.db.models import Q
from django.utils.safestring import mark_safe

from ..models import Alert

register = template.Library()


@register.simple_tag(takes_context=True)
def alerts(context, tags, *args, **kwargs):
    """
    Retrieves number of unread alerts by ``tag`` for current user from request.
    Notifications could be specified by optional argument ``subject`` as well.
    For context JSON field lookup you need to pass both ``context_attribute``
    and ``context_value`` arguments.

    Syntax::

        {% alerts 'tag' [subject=my_subject_object] [context_attribute='foo'] [context_value='bar'] %}

    Example usage::

        {% alerts 'new-comments' %}
            ... shows number of all unread comments of user articles

        {% alerts 'new-comments' subject=article %}
            ... shows number all unread comments of specified article

        {% alerts 'new-comments' context_attribute='category' context_value='100' %}
            ... shows number all unread comments of specified context
    """

    subject = kwargs.get('subject', None)
    context_attribute = kwargs.get('context_attribute', None)
    context_value = kwargs.get('context_value', None)

    if context_attribute and not context_value:
        raise ValueError(u"'context_value' argument is missing")

    if context_value and not context_attribute:
        raise ValueError(u"'context_attribute' argument is missing")

    user = context['request'].user
    tags = tags.split('|')
    alerts = Alert.objects.unread().for_recipient(user).by_tags(tags)

    if subject:
        alerts = alerts.of_subject(subject)

    if context_attribute and context_value:
        context = {context_attribute: context_value}
        alerts = alerts.in_context(context)

    number = alerts.count()

    if number > 0:
        return mark_safe('<span class="badge badge-notification">%d</span>' % number)
    return ''



from django import forms
from django.forms import ModelForm
from django.utils.translation import ugettext_lazy as _

from models import Alert
from utils import get_message_handler


class AlertForm(ModelForm):
    tag = forms.ChoiceField(label=_(u'tag'))

    class Meta:
        model = Alert

    def __init__(self, *args, **kwargs):
        super(AlertForm, self).__init__(*args, **kwargs)
        self.fields['tag'].choices = self.tag_choices()

    def tag_choices(self):
        message_handler = get_message_handler()
        return getattr(message_handler, 'TAGS', None)
